# ========================================================================
# TestReadStockPricesCSV.py. Read time-history of stock prices from a
# csv file, separate into groups (for AAPL, MSFT and GOOG), then plot. 
# 
# Written by: Mark Austin                                         May 2022
# ========================================================================

from pandas import DataFrame
from pandas import Series
from pandas import read_csv

import numpy as np 
import pandas as pd

import matplotlib.pyplot as plt

# ==============================
# main method ...
# ==============================

def main():
    print("--- Enter TestReadStockPricesiCSV.main()     ... ");
    print("--- ======================================== ... ");

    # load dataset

    print("--- ");
    print("--- Part 01: Load stock price dataset ... ");
    print("--- ");

    df = pd.read_csv("../data/stocks.csv")

    # print dataframe contents ...

    print("--- ");
    print("--- Part 02: Print dataframe contents ... ");
    print("--- ");

    print(df)

    # Filter dataframe to keep only the AAPL entries ...
    # selecting rows based on condition 
    # Assign index numbers to 0, 1, ... 122.

    print("--- ");
    print("--- Part 03: Filter dataframe to keep only AAPL entries ... ")
    print("--- ");

    options = ['AAPL'] 
    df_aapl = df [ df['symbol'].isin(options) ].copy()  
    df_aapl.index = list( range(0, 123, 1 ) )
    
    print("--- ");
    print('--- AAPL dataframe :\n', df_aapl)
    print("--- ");

    # Filter dataframe to keep only the MSFT entries ...
    # selecting rows based on condition 

    print("--- ");
    print("--- Part 04: Filter dataframe to keep only MSFT entries ... ")
    print("--- ");

    options = ['MSFT'] 
    df_msft = df [ df['symbol'].isin(options) ].copy() 
    
    print('--- MSFT dataframe :\n', df_msft)

    print("--- ");
    print("--- Part 05: Filter dataframe to keep only GOOG entries ... ")
    print("--- ");

    options = ['GOOG'] 
    df_goog = df [ df['symbol'].isin(options) ].copy() 
    df_goog.index = list( range(0, 68, 1 ) )
    
    print('--- GOOG dataframe :\n', df_goog )

    # =============================================================
    # Create line plots: MSFT, GOOG and AAPL price vs date.
    # =============================================================

    ax = plt.gca()
    df_aapl.plot(kind='line',x='date',y='price',color='blue',ax=ax)
    df_goog.plot(kind='line',x='date',y='price',color='green',ax=ax)
    df_msft.plot(kind='line',x='date',y='price',color='red',ax=ax)
    plt.show()
 
    print("--- ======================================== ... ");
    print("--- Leave TestReadStockPricesCSV.main()      ... ");

# ==================================================
# call the main method ...
# ==================================================

main()
