# =================================================================
# TestArrays.py: Exercise arrays in one, two, and three dimensions. 
# 
#                                                     November 2022 
# =================================================================

import numpy as np  # Make numpy available using np.
import numpy.matlib

# ===============================================================
# Function to print two-dimensional matrices ...
# ===============================================================

def PrintMatrix(name, a):
    print("Matrix: {:s} ".format(name) );
    for row in a:
        for col in row:
            print("{:8.3f}".format(col), end=" ")
        print("")

# ===============================================================
# main method ...
# ===============================================================

def main():
    print("--- Enter TestArrays.main()                  ... ");
    print("--- ======================================== ... ");

    # Create a numpy array, and append an element

    print("--- ")
    print("--- Part 1: Create 0-d array ...")
    print("--- ")

    A = np.array(101);
    print (A)

    print("--- ")
    print("--- Part 2: Create 1-d array of numbers ...")
    print("--- ")

    A = np.array( [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10] );
    print (A)

    print("--- ")
    print("--- Part 3: Create 1-d array of character strings ...")
    print("--- ")

    a = np.array(["The", "Quick", "Brown", "Fox"])
    a = np.append(a, "!")

    print("--- Part 3.1: Loop over array indices ... ")

    for i in a:
       print(i)

    print("--- ")
    print("--- Part 3.2: Print each element and their index:")
    print("--- ")

    for i,e in enumerate(a):
       print("Index: {}, was: {}".format(i, e))

    print("--- ")
    print("--- Part 4: Sort items in 1-d arrays ... ")
    print("--- ")

    print("--- Sort array of floating-point numbers ... ")

    a = np.array( [ 2.3, 1.0, 4.5, -13.0, 100.0, 43, -15.0, 0.0 ] )
    print(a);
    print(np.sort(a));

    print("--- Sort array of characters ... ")

    a = np.array( ['M', 'A', 'R', 'Y', 'L', 'A', 'N', 'D'] )
    print(a);
    print(np.sort(a));

    print("--- Sort array of state abbreviations ... ")

    a = np.array( ["MD", "CA", "RI", "UT", "LA", "AL", "WA", "OR", "CO"] )
    print(a);
    print(np.sort(a));

    print("--- ")
    print("--- Part 5: Show some basic math on arrays")
    print("--- ")

    b = np.array([0,1,4,3,2])
    print(b)

    print("    Max: {}".format(np.max(b)))
    print("    Average: {}".format(np.average(b)))
    print("    Max array index: {}".format(np.argmax(b)))

    print("--- ")
    print("--- Part 6: Create two-dimensional array ...");
    print("--- ")

    c = np.array( [ ( 0, 1, 4, 3, 2),
                    ( 3, 4, 5, 6, 7),
                    ( 6, 7, 8, 9,10),
                    ( 9,10,11,12,13)
                  ] );
    print(c)

    print("    Min: {}".format(np.min(c)))
    print("    Max: {}".format(np.max(c)))
    print("    Average: {}".format(np.average(c)))
    print("    Max array index: {}".format(np.argmax(c)))

    print("--- ")
    print("--- Print 2-d array as formatted matrix ...");
    print("--- ")

    PrintMatrix("C", c);

    print("--- ")
    print("--- Part 7: Create and print three-dimensional array block ...");
    print("--- ")

    c = np.array( [ [ ( 0, 1), (3, 4) ], [(5, 6), (7, 8) ] ] );
    print(c)

    print("--- ")
    print("--- Part 8: Reshape arrays ...");
    print("--- ")

    print("--- Reshape 1-d array: d1 = np.arange(20).reshape(4,5) ...");
    print("--- 1-d test array: ")

    d1 = np.arange(20);
    print(d1);

    print("--- Reshape to (4x5) matrix ...")

    d1 = d1.reshape(4,5);
    PrintMatrix("(4x5)", d1 );

    print("--- ")
    print("--- Part 9: Create horizontal and vertical array stacks ...");
    print("--- ")

    d1 = np.array( [ ( 0, 1), ( 3, 4) ] );
    d2 = np.array( [ ( 5, 6), ( 7, 8) ] );

    PrintMatrix("d1", d1 );
    PrintMatrix("d2", d2 );

    print("--- Horizontal array stack ...")

    h1 = np.hstack((d1, d2));
    PrintMatrix( "np.hstack(d1, d2)", h1 );

    print("--- Vertical array stack ...")

    h2 = np.vstack((d1, d2));
    PrintMatrix( "np.vstack(d1, d2)", h2 );

    print("--- ======================================== ... ");
    print("--- Leave TestArrays.main()                  ... ");

# call the main method ...

main()
