# ===============================================================
# TestTensors.py: Exercise tensor operations in TensorFlow 2. 
# 
# Written by: Mark Austin                              April 2021 
# ===============================================================

import tensorflow as tf
from tensorflow.python.framework import ops

# main method ...

def main():
    print("--- Enter TestTensors.main()                 ... ");
    print("--- ======================================== ... ");
    print("--- ");
    print("--- Basic Tensor ... ");
    print("--- ");

    # Introduce tensors in tf
 
    my_tensor = tf.zeros([1, 5])
    print(my_tensor)

    print("--- ");
    print("--- Declare and print variables ... ");
    print("--- ");

    # Declare a variable

    my_var = tf.Variable(tf.zeros([1, 20]))
    print(my_var)

    # Different kinds of variables

    print("--- ");
    print("--- Declare different kinds of variables ... ");
    print("--- ");

    row_dim = 2
    col_dim = 3

    # Zero initialized variable

    zero_var = tf.Variable(tf.zeros([row_dim, col_dim]))
    print(zero_var)

    # One initialized variable

    ones_var = tf.Variable(tf.ones([row_dim, col_dim]))
    print(ones_var)

    # Create a variable from a constant

    const_var = tf.Variable(tf.constant([8, 6, 7, 5, 3, 0, 9]))
    print(const_var)

    # This can also be used to fill an array:

    const_fill_var = tf.Variable(tf.constant(-1, shape=[row_dim, col_dim]))
    print(const_fill_var)

    print("--- ");
    print("--- Create and print rank two tensor ... ");
    print("--- ");

    # If you want to be specific, you can set the dtype (see below) at creation time

    rank_2_tensor = tf.constant([[1, 2],
                             [3, 4],
                             [5, 6]], dtype=tf.float16)
    print(rank_2_tensor)

    # There can be an arbitrary number of axes (sometimes called "dimensions")

    print("--- ");
    print("--- Create and print rank three tensor ... ");
    print("--- ");

    rank_3_tensor = tf.constant([ [[0, 1, 2, 3, 4],
                                   [5, 6, 7, 8, 9]],
                              [[10, 11, 12, 13, 14],
                               [15, 16, 17, 18, 19]],
                              [[20, 21, 22, 23, 24],
                               [25, 26, 27, 28, 29]],] )

    print(rank_3_tensor)

    print("--- ");
    print("--- Declare and print sequences ... ");
    print("--- ");

    # Sequence: generates [0.0, 0.5, 1.0] includes the end

    linear_var = tf.Variable(tf.linspace(start=0.0, stop=1.0, num=3))
    print(linear_var)

    # Sequence: generates [6, 9, 12], but doesn't include the end

    sequence_var = tf.Variable(tf.range(start=6, limit=15, delta=3))
    print(sequence_var)

    print("--- ");
    print("--- Basic math on tensors ... ");
    print("--- ");

    a = tf.constant([[1, 2], [3, 4]])
    b = tf.constant([[1, 1], [1, 1]]) # Could have also said `tf.ones([2,2])`

    print( tf.add(a, b), "\n")
    print( tf.multiply(a, b), "\n")
    print( tf.matmul(a, b), "\n")

    print("--- ");
    print("--- Element-wise addition and multiplication ... ");
    print("--- ");

    print(a + b, "\n") # element-wise addition
    print(a * b, "\n") # element-wise multiplication
    print(a @ b, "\n") # matrix multiplication

    print("--- ");
    print("--- Arbitrary operations on tensors ... ");
    print("--- ");

    # Find the largest value

    print("--- Test tensor c ...\n");

    c = tf.constant([[4.0, 5.0], [10.0, 1.0]])
    print(c)

    print("--- Arbitrary tensor operations ...\n");

    # Find the largest value

    print( tf.reduce_max(c) )

    # Find the index of the largest value

    print( tf.math.argmax(c) )

    # Compute the softmax

    print( tf.nn.softmax(c) )

    print("--- ======================================== ... ");
    print("--- Finished!! ... ");

# call the main method ...

main()
